@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.142 2014/10/08 17:27:06 stroeder Exp $
+\$Id: ldapobject.py,v 1.146 2015/06/11 15:13:43 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -32,7 +32,7 @@ if __debug__:
   # Tracing is only supported in debugging mode
   import traceback
 
-import sys,time,pprint,_ldap,ldap,ldap.functions
+import sys,time,pprint,_ldap,ldap,ldap.sasl,ldap.functions
 
 from ldap.schema import SCHEMA_ATTRS
 from ldap.controls import LDAPControl,DecodeControlTuples,RequestControlTuples
@@ -136,6 +136,14 @@ class SimpleLDAPObject:
         self.__class__.__name__,repr(name)
       )
 
+  def fileno():
+    """
+    Returns file description of LDAP connection.
+
+    Just a convenience wrapper for LDAPObject.get_option(ldap.OPT_DESC)
+    """
+    return self.get_option(ldap.OPT_DESC)
+
   def abandon_ext(self,msgid,serverctrls=None,clientctrls=None):
     """
     abandon_ext(msgid[,serverctrls=None[,clientctrls=None]]) -> None
@@ -235,6 +243,30 @@ class SimpleLDAPObject:
     """
     return self._ldap_call(self._l.sasl_interactive_bind_s,who,auth,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls),sasl_flags)
 
+  def sasl_non_interactive_bind_s(self,sasl_mech,serverctrls=None,clientctrls=None,sasl_flags=ldap.SASL_QUIET):
+    """
+    Send a SASL bind request using a non-interactive SASL method (e.g. GSSAPI, EXTERNAL)
+    """
+    self.sasl_interactive_bind_s('',ldap.sasl.sasl({},sasl_mech))
+
+  def sasl_external_bind_s(self,serverctrls=None,clientctrls=None,sasl_flags=ldap.SASL_QUIET):
+    """
+    Send SASL bind request using SASL mech EXTERNAL
+    """
+    self.sasl_non_interactive_bind_s('EXTERNAL',serverctrls,clientctrls,sasl_flags)
+
+  def sasl_gssapi_bind_s(self,serverctrls=None,clientctrls=None,sasl_flags=ldap.SASL_QUIET):
+    """
+    Send SASL bind request using SASL mech GSSAPI
+    """
+    self.sasl_non_interactive_bind_s('GSSAPI',serverctrls,clientctrls,sasl_flags)
+
+  def sasl_bind_s(self,dn,mechanism,cred,serverctrls=None,clientctrls=None):
+    """
+    sasl_bind_s(dn, mechanism, cred [,serverctrls=None[,clientctrls=None]]) -> int|str
+    """
+    return self._ldap_call(self._l.sasl_bind_s,dn,mechanism,cred,RequestControlTuples(serverctrls),RequestControlTuples(clientctrls))
+
   def compare_ext(self,dn,attr,value,serverctrls=None,clientctrls=None):
     """
     compare_ext(dn, attr, value [,serverctrls=None[,clientctrls=None]]) -> int
@@ -648,7 +680,7 @@ class SimpleLDAPObject:
   def read_s(self,dn,filterstr=None,attrlist=None,serverctrls=None,clientctrls=None,timeout=-1):
     """
     Reads and returns a single entry specified by `dn'.
-    
+
     Other attributes just like those passed to `search_ext_s()'
     """
     r = self.search_ext_s(
@@ -751,6 +783,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     '_l':None,
     '_ldap_object_lock':None,
     '_trace_file':None,
+    '_reconnect_lock':None,
   }
 
   def __init__(
@@ -789,6 +822,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     """set up the object from pickled data"""
     self.__dict__.update(d)
     self._ldap_object_lock = self._ldap_lock()
+    self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
     self._trace_file = sys.stdout
     self.reconnect(self._uri)
 
@@ -893,6 +927,11 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._store_last_bind(SimpleLDAPObject.sasl_interactive_bind_s,*args,**kwargs)
     return res
 
+  def sasl_bind_s(self,dn,mechanism,cred,serverctrls=None,clientctrls=None):
+    res = self._apply_method_s(SimpleLDAPObject.sasl_bind_s,*args,**kwargs)
+    self._store_last_bind(SimpleLDAPObject.sasl_bind_s,*args,**kwargs)
+    return res
+
   def add_ext_s(self,*args,**kwargs):
     return self._apply_method_s(SimpleLDAPObject.add_ext_s,*args,**kwargs)
 
